import 'dart:typed_data';
import 'credentials.dart';

enum EncryptMethod { SM2, SECP256K1 }

const String KEY_ADDR = 'key:addr';
const String KEY_PRIVATE = 'key:private';
const String KEY_PUBLIC = 'key:public';

class EncryptoMessage {
  String publicKey;
  Uint8List body;
  EncryptoMessage(this.publicKey, this.body);
}

abstract class ISharePreferences {
  String getString(String key);
  void setString(String key, String value);
}

class CoreIdentity {
  static final CoreIdentity _singleton = new CoreIdentity._internal();

  factory CoreIdentity() {
    return _singleton;
  }

  CoreIdentity._internal();

  Cipher cipher;
  //Future<SharedPreferences> _prefs = SharedPreferences.getInstance();
  StringKeypair accountKp;
  ISharePreferences _store;

  CoreIdentity setup(EncryptMethod method, ISharePreferences storage) {
    if (method == EncryptMethod.SM2) {
      this.cipher = SM2Cipher();
    }
    if (method == EncryptMethod.SECP256K1) {
      this.cipher = SECPCipher();
    }
    this._store = storage;
    return this;
  }

  Future init() async {
    final addr = _store.getString(KEY_ADDR) ?? '';
    if (addr.isEmpty) {
      this.accountKp = await this.cipher.makePair();
      _store.setString(KEY_ADDR, this.accountKp.address);
      _store.setString(KEY_PUBLIC, this.accountKp.publicKey);
      _store.setString(KEY_PRIVATE, this.accountKp.privateKey);
    }
  }

  Future restoreAccount(String privateKey) async {
    final stringKeypair = await this.cipher.restorePair(privateKey);
    this._store.setString(KEY_ADDR, stringKeypair.address);
    this._store.setString(KEY_PUBLIC, stringKeypair.publicKey);
    this._store.setString(KEY_PRIVATE, stringKeypair.privateKey);
  }

  Future<StringKeypair> account() async {
    return StringKeypair(this._store.getString(KEY_ADDR),
        this._store.getString(KEY_PRIVATE), this._store.getString(KEY_PUBLIC));
  }

  Future<EncryptoMessage> sendTo(
      String targetPublickey, Uint8List payload) async {
    final kp = await this.cipher.makePair();

    final body = await this
        .cipher
        .privateEncrypto(kp.privateKey, targetPublickey, payload);
    return EncryptoMessage(kp.publicKey, body);
  }

  Future<Uint8List> decodeMessage(EncryptoMessage message) async {
    final privateKey = this._store.getString(KEY_PRIVATE);
    //final privateKey = this.accountKp.privateKey;
    return await this
        .cipher
        .publicDecrypto(privateKey, message.publicKey, message.body);
  }

  Future<Uint8List> processFile(Uint8List fileData, Uint8List pwd, Uint8List iv) async{
    return await this.cipher.prcessCrypto(pwd, iv, fileData);
  }

}
